use std::path;

use anyhow::Result;
use clap::{Parser, Subcommand};

/// Generate register interface from register map xml.
#[derive(Parser, Debug)]
#[command(version, about)]
struct Args {
    /// Input XML register map.
    xml: path::PathBuf,
    #[command(subcommand)]
    command: Commands,
}

#[derive(Debug, Subcommand)]
enum Commands {
    /// Generate register interface code.
    Generate {
        /// Output directory.
        out: path::PathBuf,
    },
    /// Generate flattened register map in CSV.
    #[cfg(feature = "flatmap")]
    Flatmap {
        /// Flattened csv out path.
        /// Print to stdout by default.
        flatmap: Option<path::PathBuf>,
    },
}

fn main() -> Result<()> {
    env_logger::init();
    log::debug!("logger enabled");
    let args = Args::parse();
    log::debug!("args: {:?}", args);

    match args.command {
        Commands::Generate { out } => endcap_sl_software_ri_generator::generate(&args.xml, &out)?,
        #[cfg(feature = "flatmap")]
        Commands::Flatmap { flatmap } => {
            let outpath: Option<&path::Path> = match flatmap {
                Some(ref p) => Some(p.as_path()),
                None => None,
            };
            endcap_sl_software_ri_generator::integrated::generate_flatmap(&args.xml, outpath)?
        }
    }

    Ok(())
}
